package main

import (
	"log"
	"os"
	"strconv"

	"git.pbiernat.dev/egommerce/basket-service/internal/app/config"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/database"
	"git.pbiernat.dev/egommerce/basket-service/internal/app/server"
	"git.pbiernat.dev/egommerce/basket-service/pkg/amqp"
	"git.pbiernat.dev/egommerce/basket-service/pkg/fluentd"
)

const (
	defAppName       = "basket-svc"
	defAppDomain     = "basket-svc"
	defNetAddr       = ":80"
	defLoggerAddr    = "api-logger:24224"
	defDbURL         = "postgres://postgres:12345678@postgres-db:5432/egommerce"
	defMongoDbURL    = "mongodb://mongodb:12345678@mongo-db:27017"
	defEventBusURL   = "amqp://guest:guest@api-eventbus:5672"
	ebEventsExchange = "api-events"
	ebEventsQueue    = "basket-svc"
)

func main() {
	if config.ErrLoadingEnvs != nil {
		log.Panicln("Error loading .env file", config.ErrLoadingEnvs)
	}

	c := new(server.Config)
	c.AppName = config.GetEnv("APP_NAME", defAppName)
	c.AppDomain = config.GetEnv("APP_DOMAIN", defAppDomain)
	c.NetAddr = config.GetEnv("SERVER_ADDR", defNetAddr)
	c.Port, _ = strconv.Atoi(c.NetAddr[1:])
	c.LoggerAddr = config.GetEnv("LOGGER_ADDR", defLoggerAddr)
	c.DbURL = config.GetEnv("DATABASE_URL", defDbURL)
	c.EventBusURL = config.GetEnv("EVENTBUS_URL", defEventBusURL)
	c.EventBusExchange = ebEventsExchange

	logHost, logPort := fluentd.ParseAddr(c.LoggerAddr)
	logger := fluentd.NewLogger(c.AppName, logHost, logPort)
	defer logger.Close()

	// db conn
	dbConn, err := database.Connect(c.DbURL)
	if err != nil { // fixme: add wait-for-db...
		logger.Log("Failed to connect to Database server: %v\n", err)
		os.Exit(1)
	}
	defer dbConn.Close()

	// eventbus conn
	ebConn, ebCh, err := amqp.Open(c.EventBusURL)
	if err != nil {
		logger.Log("Failed to connect to EventBus server: %v\n", err)
		os.Exit(1)
	}
	defer ebCh.Close()
	defer amqp.Close(ebConn)

	err = amqp.NewExchange(ebCh, c.EventBusExchange)
	if err != nil {
		logger.Log("Failed to declare EventBus exchange: %v\n", err)
		os.Exit(1)
	}

	// start server
	srv := server.NewServer(c, logger, dbConn, ebCh)
	srv.StartWithGracefulShutdown()
}
